//block_array.h: A header file for an object that is an array
//of blocks, used to form a CROWBAR

#if !defined(_Block_Array_h_)
#define _Block_Array_h_

///////////////////////////////////////////////////////////////////////////////

#include <iostream.h>
#include "signal.h"
#include "cell.h"
#include "blocks.h"

//A Crowbar is made up of 12 interconnected blocks

// S0 L0 L1 S1
// L2 L3 L4 L5
// S2 L6 L7 S3

#define PINS 40
#define SENSOR_BLOCKS 4
#define LOGIC_BLOCKS 8
#define PIN_NAME_SIZE 16

///////////////////////////////////////////////////////////////////////////////
//Pin assignments (start counting with zero)

#define UNDEFINED_PIN -1
#define L5NIN_H    0
#define S1OUT_H    1
#define S1NIN_H    2
#define S1WIN_H    3
#define PAD_VDD1   4
#define SCANIN_H   5
#define RST_L      6
#define L1WIN_H    7
#define L1OUT_H    8
#define GND        9
#define CLK_A     10
#define L0WIN_H   11
#define L0OUT_H   12
#define CONFOUT_H 13
#define PAD_VDD2  14
#define S0WIN_H   15
#define S0OUT_H   16
#define S0SIN_H   17
#define SCANOUT_H 18
#define L2SIN_H   19
#define L2OUT_H   20
#define S2OUT_H   21
#define S2SIN_H   22
#define CONFIN_H  23
#define PAD_GND1  24
#define TEST_H    25
#define S2EIN_H   26
#define L6EIN_H   27
#define L6OUT_H   28
#define VDD       29
#define CLK_B     30
#define L7EIN_H   31
#define L7OUT_H   32
#define CONFOPT_H 33
#define PAD_GND2  34
#define ENCONF_H  35
#define S3EIN_H   36
#define S3OUT_H   37
#define S3NIN_H   38
#define L5OUT_H   39

typedef int pin_name_type; 
void Pin_Name_To_String(pin_name_type pin_name, char* string);
pin_name_type String_To_Pin_Name(char* string); //return -1 for error

///////////////////////////////////////////////////////////////////////////////
//Sensor assignments

#define UNDEFINED_CSB -1
#define CSB0 0
#define CSB1 1
#define CSB2 2
#define CSB3 3

typedef int csb_name_type;
csb_name_type String_To_CSB_Name(char* string); //return -1 for error

///////////////////////////////////////////////////////////////////////////////
//next we'll define an object for a complete CROWBAR array


class CROWBAR_class 
{
 public:

  CROWBAR_class(); //constructor
  ~CROWBAR_class(); //destructor
  void Clear(); //clear array data structures
  void Update(unsigned int gate_delays); //can take several delays to converge 
  void Apply_Voltage_To_Pin(pin_name_type pin_name, voltage_type voltage);
  void Apply_Light_To_Sensor(unsigned int sensor_number, optical_type light);
  voltage_type Get_Pin_Voltage(pin_name_type pin_name); //by name
  void Print_Flop_Outputs(); //handy for debugging
  void Print_Block_Outputs(); //handy for debugging
  void Print_Config(); //handy for debugging
  void Print_Scan();
  void Print_Pin_Values(); //handy for debugging
  void Print_A_Pin_Value(pin_name_type pin_name);
  void Print_Image(); //handy for debugging
  void Print_Pinout(); //handy

 protected:

  //protected data
  CSB_class Sensor_block_list[SENSOR_BLOCKS]; 
  CLB_class Logic_block_list[LOGIC_BLOCKS];
  Pin_class Pin_list[PINS];

  //protected functions
  bool Array_Converged(); //did all signals converge?
  void Update(); //update state of array (1 gate delay)
  signal_type Get_Pin_Signal(pin_name_type pin_name); //by name
  void Apply_Signal_To_Pin(pin_name_type pin_name, signal_type signal);

 private:

  //private data
  optical_type image[4]; //current image on chip

  //private functions
  bool Check_Inputs(); //Check chip inputs for stupid stuff (print errors)
  //return false for a fatal error, print warnings otherwise
  void Print_CSB_Config_Bits(unsigned int CSB_number); //macros are good
  void Print_CLB_LUT_Config_Bits(unsigned int CLB_number);
  void Print_CLB_Mux_Config_Bits(unsigned int CLB_number);
}; //CROWBAR_class

///////////////////////////////////////////////////////////////////////////////
#endif






