//interface.cc: implementation file for sim-crowbar interface
//state machine code

#include "interface.h"
#include "block_array.h"
#include "signal.h"
#include <iostream.h>
#include "string.h"

//extern CROWBAR_class* chip;

///////////////////////////////////////////////////////////////////////////////
//Parser routines

state_type next_state(token_type token, token_data data, state_type old_state)
{
  state_type new_state = old_state;
  parse_error_type error_code;
  bool lambda = true; //lambda transitions between states
  static voltage_type voltage = Floating;
  static optical_type optic = Undefined;
 
  while(lambda == true)
    { 
      switch(new_state)
	{
	case START_STATE:
	  lines = 0;
	  chip->Clear();
	  print_intro();
	  new_state = PROMPT_STATE;
	  break;
	case PROMPT_STATE:
	  prompt_for_input(++lines);
	  new_state = WAIT_FOR_COMMAND;
	  break;
	case WAIT_FOR_COMMAND:
	  switch(token)
	    {
	    case ASSIGN_PIN:
	      new_state = WAIT_FOR_PIN;
	      switch(data.command)
		{
		case 'h': voltage = High; 
		  break;
		case 'l': voltage = Low; 
		  break;
		default: voltage = Floating;
		} //switch
	      break;
	    case ASSIGN_OPTICAL:
	      new_state = WAIT_FOR_OPTIC;
	      break;
	    case QUIT:
	      new_state = CONFIRM_QUIT;
	      break;
	    case PRINT:
	      new_state = WAIT_FOR_OPTION;
	      break;
	    case SIMULATE:
	      new_state = WAIT_FOR_GATE_DELAYS;
	      break;
	    case CONFIG_STRING:
	      new_state = CONFIGURE;
	      break;
	    case ABOUT:
	      new_state = ABOUT_STATE;
	      break;
	    case HELP:
	      new_state = HELP_STATE;
	      break;
	    case NAME:
	    case POS_INT:
	    case YES:
	    case NO:
	    case ERROR:
	      error_code = UNDEFINED_COMMAND;
	      new_state = ERROR_STATE;
	      break;
	    case END:
	      new_state = PROMPT_STATE;
	      break;
	    default:
	      new_state = WAIT_FOR_COMMAND;
	    } //switch
	  break;
	case CONFIGURE:
	  for(unsigned int i = 0; i<strlen(data.config); i++)
	    {
	      check_and_shift_config(data.config[i]);
	    } //for
	  new_state = PROMPT_STATE;
	  break;
	case WAIT_FOR_PIN:
	  switch(token)
	    {
	    case NAME:
	      new_state = WAIT_FOR_ANOTHER_PIN;
	      check_and_assign_pin(data.name, voltage);
	      break;
	    case POS_INT:
	      new_state = WAIT_FOR_ANOTHER_PIN;
	      check_and_assign_pin((pin_name_type)data.ival, voltage);
	      break;
	    default:
	      error_code = INVALID_PIN;
	      new_state = ERROR_STATE;
	    } //switch
	  break;
	case WAIT_FOR_ANOTHER_PIN:
	  switch(token)
	    {
	    case NAME:
	      new_state = WAIT_FOR_ANOTHER_PIN;
	      check_and_assign_pin(data.name, voltage);
	      break;
	    case POS_INT:
	      new_state = WAIT_FOR_ANOTHER_PIN;
	      check_and_assign_pin((pin_name_type)data.ival, voltage);
	      break;
	    case END:
	      new_state = PROMPT_STATE;
	      break;
	    default:
	      error_code = INVALID_PIN;
	      new_state = ERROR_STATE;
	    } //switch
	  break;
	case WAIT_FOR_OPTIC:
	  switch(token)
	    {
	    case POS_INT:
	      optic = (optical_type)data.ival;
	      new_state = WAIT_FOR_SENSOR;
	      break;
	    default:
	      error_code = INVALID_OPTIC;
	      new_state = ERROR_STATE;
	    } //switch
	  break;
	case WAIT_FOR_GATE_DELAYS:
	  switch(token)
	    {
	    case POS_INT:
	      check_and_simulate(data.ival);
	      new_state = WAIT_FOR_EOL;
	      break;
	    default:
	      error_code = INVALID_GATE_DELAYS;
	      new_state = ERROR_STATE;
	    } //switch
	  break;
	case WAIT_FOR_SENSOR:
	  switch(token)
	    {
	    case NAME:
	      check_and_assign_optical(data.name, optic);
	      new_state = WAIT_FOR_ANOTHER_SENSOR;
	      break;
	    case POS_INT:
	      check_and_assign_optical(data.ival, optic);
	      new_state = WAIT_FOR_ANOTHER_SENSOR;
	      break;
	    case END: 
	      new_state = PROMPT_STATE;
	      break;
	    default:
	      error_code = INVALID_NAME;
	      new_state = ERROR_STATE;
	    } //switch
	  break;
	case WAIT_FOR_ANOTHER_SENSOR:
	  switch(token)
	    {
	    case NAME:
	      check_and_assign_optical(data.name, optic);
	      new_state = WAIT_FOR_ANOTHER_SENSOR;
	      break;
	    case POS_INT:
	      check_and_assign_optical(data.ival, optic);
	      new_state = WAIT_FOR_ANOTHER_SENSOR;
	      break;
	    default:
	      error_code = INVALID_NAME;
	      new_state = ERROR_STATE;
	    } //switch
	  break;
	case WAIT_FOR_OPTION:
	  switch(token)
	    {
	    case NAME:
	      check_and_print_data(data.name);
	      new_state = WAIT_FOR_ANOTHER_OPTION;
	      break;
	    default:
	      error_code = INVALID_OPTION;
	      new_state = ERROR_STATE;
	    } //switch
	  break;	
	case WAIT_FOR_ANOTHER_OPTION:
	  switch(token)
	    {
	    case NAME:
	      check_and_print_data(data.name);
	      new_state = WAIT_FOR_ANOTHER_OPTION;
	      break;
	    case END:
	      new_state = PROMPT_STATE;
	      break;
	    default:
	      error_code = INVALID_OPTION;
	      new_state = ERROR_STATE;
	    } //switch
	  break;
	case ABOUT_STATE:
	  print_about();
	  new_state = PROMPT_STATE;
	  break;
	case HELP_STATE:
	  print_help();
	  new_state = PROMPT_STATE;
	  break;
	case WAIT_FOR_EOL:
	  switch(token)
	    {
	    case END:
	      new_state = PROMPT_STATE;
	      break;
	    default:
	      error_code = EOL_EXPECTED;
	      new_state = ERROR_STATE;
	    } //switch
	  break;
	case ERROR_STATE:
	  //chew up the rest of the line
          print_error(error_code, lines);
	  switch(token)
	    {
	    case END:
	      new_state = PROMPT_STATE; //that was it
	      break;
	    default:
	      new_state = DISCARD_LINE; //other stuff
	    }
	  break;
	case DISCARD_LINE:
	  switch(token)
	    {
	    case END:
	      new_state = PROMPT_STATE;
	      break;
	    default:
	      ;
	    } //switch
	  break;
	case CONFIRM_QUIT:
          prompt_for_confirmation();
	  new_state = WAIT_FOR_RESPONSE;
	  break;
	case WAIT_FOR_RESPONSE:
	  switch(token)
	    {
	    case YES:
	      new_state = END_STATE;
	      break;
	    case NO:
	      new_state = PROMPT_STATE;
	      break;
	    default:
	      error_code = INVALID_RESPONSE;
	      new_state = ERROR_STATE;
	    } //switch
	  break;
	default:
	  new_state = PROMPT_STATE;
	} //switch old_state
 
      switch(new_state)
	{ 
	case WAIT_FOR_COMMAND:
	case WAIT_FOR_PIN:
	case WAIT_FOR_ANOTHER_PIN:
	case WAIT_FOR_OPTIC:
	case WAIT_FOR_GATE_DELAYS:
	case WAIT_FOR_SENSOR:
	case WAIT_FOR_ANOTHER_SENSOR:
	case WAIT_FOR_OPTION:
	case WAIT_FOR_ANOTHER_OPTION:
	case WAIT_FOR_RESPONSE:
	case WAIT_FOR_EOL:
	case DISCARD_LINE:
        case END_STATE:
	  lambda = false;
	  break;
	default:
	  lambda = true;
	} //switch
    }//while
  return new_state;
} //state_machine


void print_intro()
{
 cout<<"Welcome to Sim-Crowbar:"<<endl;
 cout<<"The interactive simulator for the Crowbar Prototype"<<endl;
 cout<<"Type 'help' for a list of commands"<<endl;
 cout<<"Type 'about' for more information"<<endl;
} //print_intro


void print_error(parse_error_type code, counter_type line_number)
{
  cerr<<"***Parse error: line "<<line_number<<", ";
  switch(code)
    {
    case UNDEFINED_COMMAND:
      cerr<<"Undefined command"<<endl;
      break;
    case INVALID_PIN:
      cerr<<"Invalid pin argument"<<endl;
      break;
    case INVALID_OPTIC:
      cerr<<"Invalid optical intensity"<<endl;
      break;
    case INVALID_GATE_DELAYS:
      cerr<<"Invalid gate delays"<<endl;
      break;
    case INVALID_NAME:
      cerr<<"Invalid name"<<endl;
      break;
    case INVALID_OPTION:
      cerr<<"Invalid print option"<<endl;
      break;
    case EOL_EXPECTED:
      cerr<<"Characters found where end-of-line expected"<<endl;
      break;
    case INVALID_RESPONSE:
      cerr<<"Invalid response"<<endl;
      break;
    default:
      cerr<<"Error type unknown"<<endl;
    } //switch
  cerr<<"\tType 'help' for usage"<<endl;
} //print_error


void prompt_for_input(counter_type line_number)
{
   cout<<"sim-crowbar("<<line_number<<"):";
} //prompt_for_input


void prompt_for_confirmation()
{
 cout<<"Quitting: Are you sure? (y/n)\n";
} //prompt_for_confirmation


void print_about()
{
  char message[] = "
Crowbar was a project at the University of Cincinnati aimed
at integrating optics into a sea-of-gates FPGA. A full-custom
prototype was designed and fabricated in 2000 by Jason Cantin
and Dr. Fred Beyette Jr. In addition to simple logic blocks,
the FPGA contains four blocks that are equipped with simple
optical detectors (each with a programmable threshold).

This simulator was developed in early 2001 at the University
of Wisconsin-Madison to aid in the testing of the hardware,
application development, and compiler development. It is 
fully object-oriented, and models the prototype with a very
high level of detail. Except for some handy macros, and the
lack of a waveform viewer, the interface is similar to IRSIM.

Latest Features:

Command added to allow configuration in one step. Chip is 
powered up, and the user supplied bits are shifted on 
automatically. 

Caveats: 

The simulator does not attempt to model the hysteresis of
the optical receivers. This capability can be added if this
property is measured on the real hardware.

The simulator models known bugs in the design, including the
feedback bug discovered in the design of the sensor-blocks.

The simulator has no knowledge of the effects of optical
energy on the logic circuits. It assumes no effect.

The simulator is NOT a circuit simulator, and thus does not
know about clock skew or wire delays. Wires have no delay.
Each logic block is assigned a unit delay of one. This delay 
is invariant of the function, so detailed timing analyses
are not possible.

The simulator will warn about dangerous conditions that it
checks for, and proceed. For example, both clocks high or
clk_b and reset simultaneously asserted. In such cases 
'Unknown' values will be assigned to the outputs of circuits 
for which the logical value is unclear.  

The simulator requires all power supply pins to be properly
driven. It checks for this, but will not proceed otherwise 
(It does not try to figure out what would happen in this
case). This can be changed so that the model sets all 
signals to 'Unknown' when the chip is powered down.

Jason F. Cantin, 1/2001";
cout<<message<<endl<<endl;
} //print_about


void print_help()
{
  cout<<endl;
  cout<<"***Valid commands for Sim-Crowbar***"<<endl;
  cout<<endl;
  cout<<"( l[ow] | h[igh] | x ) <pin>{ <pin>}"<<endl;
  cout<<"\twhere <pin> ::= ( <pin_name> | <pin_number> )"<<endl;
  cout<<"\twhere <pin_name> is a valid pin name (case insensitive)"<<endl;
  cout<<"\twhere <pin_number> is a positive integer between 0 and 39"<<endl;
  cout<<endl;
  cout<<"\tSet corresponding pin voltage(s) low, high, or floating."<<endl;
  cout<<endl;
  cout<<"o[ptical] <intensity> <sensor_name>{<sensor_name>}"<<endl;
  cout<<endl;
  cout<<"\tApply optical energy to named sensor(s) input."<<endl;
  cout<<"\tIntensities range from 0-7 in this implementation."<<endl;
  cout<<endl;
  cout<<"s[imulate] <gate-delays>"<<endl;
  cout<<endl;
  cout<<"\tSimulates logic for (at most) specified number of delays."<<endl;
  cout<<"\tStops if signals converge before that time."<<endl;
  cout<<endl;
  cout<<"c[onfigure] <bit>{(<bit>|<white_space>)}"<<endl;
  cout<<"\twhere <bit> ::= ( '1' | '0' )"<<endl;
  cout<<endl;
  cout<<"\tPowers up the model, and shifts bits onto config-string."<<endl;
  cout<<"\t--Much easier than manually manipulating the signals."<<endl;
  cout<<endl;
  cout<<"p[rint] <option>{ <option>}"<<endl;
  cout<<"\twhere <option> ::= (pins | config | image | flops | blocks | scan)"<<endl;
  cout<<endl;
  cout<<"\tPrints indicated information about state of chip:"<<endl;
  cout<<"\t\t'pins'   -> pin names, internal values, and pad voltages"<<endl;
  cout<<"\t\t'config' -> contents of internal config chain"<<endl;
  cout<<"\t\t'image'  -> current image seen by detectors"<<endl;
  cout<<"\t\t'flops'  -> outputs of all flip-flops on chip"<<endl;
  cout<<"\t\t'blocks' -> outputs of all blocks on chip"<<endl;
  cout<<"\t\t'scan'   -> contents of internal scan chain"<<endl;
  cout<<"\t\t'pinout' -> prints a pinout for the chip"<<endl;
  cout<<endl;
  cout<<"help"<<endl;
  cout<<"\tPrints this message."<<endl;
  cout<<endl;
  cout<<"q[uit]"<<endl;
  cout<<"\tPrompts for confirmation and quits the simulator."<<endl;
  cout<<endl;
  cout<<"Comments begin with a vertical bar ('|'), and terminate"<<endl;
  cout<<"at the end of the line."<<endl; 
  cout<<endl;
} //print_help

///////////////////////////////////////////////////////////////////////////////
//Semantic routines

void check_and_assign_pin(char* name, voltage_type voltage)
{
  pin_name_type pin_name = String_To_Pin_Name(name);
  check_and_assign_pin(pin_name, voltage);
} //check_and_assign_pin


void check_and_assign_pin(pin_name_type pin_name, voltage_type voltage)
{
  semantic_error_type error;
  if((pin_name <= UNDEFINED_PIN)||(pin_name >= PINS))
    {
      error = PIN_NUMBER_OUT_OF_RANGE;
      print_error(error, lines);
    } //if
  else
    {
      //name okay, assign the voltage
      chip->Apply_Voltage_To_Pin(pin_name, voltage);
    } //else
} //check_and_assign_pin


void check_and_print_data(char* option)
{
  semantic_error_type error;
  if(strcasecmp("pins", option)==0)
    chip->Print_Pin_Values();
  else
    if(strcasecmp("config", option)==0)
      {
       chip->Print_Config();
      }
    else
      if(strcasecmp("flops", option)==0)
	chip->Print_Flop_Outputs();
      else
	if(strcasecmp("image", option)==0)
	  chip->Print_Image();
	else
	  if(strcasecmp("blocks", option)==0)
	    chip->Print_Block_Outputs();
	  else
	    if(strcasecmp("scan", option)==0)
	      chip->Print_Scan();
	    else
	      if(strcasecmp("pinout", option)==0)
		chip->Print_Pinout();
              else
		{
		  error = UNKNOWN_OPTION;
		  print_error(error, lines);
		} //else
} //check_and_print_data


void check_and_simulate(unsigned int gate_delays)
{
  chip->Update(gate_delays);
} //check_and_simulate


void check_and_assign_optical(char* name, optical_type optic)
{
  //check name
  semantic_error_type error;
  csb_name_type number = String_To_CSB_Name(name);
  if(number == UNDEFINED_CSB)
    {
      error = CSB_NUMBER_OUT_OF_RANGE;
      print_error(error, lines);
    }
  else
    {
      check_and_assign_optical(number, optic);
    } //else
}  //check_and_assign_optical


void check_and_assign_optical(int num, optical_type optic)
{
  semantic_error_type error;
  //first check value of intensity
  if((optic<0)||(optic>7))
    {
      error = INTENSITY_OUT_OF_RANGE;
      print_error(error, lines);
    } //if
  else
    {
      chip->Apply_Light_To_Sensor(num, optic);
    }
} //check_and_assign_optical


void print_error(semantic_error_type code, counter_type line_number)
{
  cerr<<"***Semantic error: line "<<line_number<<", ";
  switch(code)
    {
    case PIN_NUMBER_OUT_OF_RANGE:
      cerr<<"Unknown Pin"<<endl;
      break;
    case INTENSITY_OUT_OF_RANGE:
      cerr<<"Intensity out of range, must be 0-7"<<endl;
      break;
    case UNKNOWN_OPTION:
      cerr<<"Unknown command option specified"<<endl;
      break;
    case CSB_NUMBER_OUT_OF_RANGE:
      cerr<<"Unknown Sensor"<<endl;
      break;
    case NOT_A_BIT:
      cerr<<"A bit is either zero or one"<<endl;
      break;
    default:
      cerr<<"Error type unknown"<<endl;
    } //switch
  cerr<<"\tType 'help' for usage"<<endl;
} //print_error


void check_and_shift_config(char bit)
{
  //handy way to configure chip in one step
  //check for 1 or 0, and shift the bit onto the chip
  const int delays = 20;
  semantic_error_type error;
  if((bit == '1')||(bit == '0'))
    {
      //power up (if not here, lots of messages if user doesn't)
      //chip->Apply_Voltage_To_Pin(VDD, High);
      //chip->Apply_Voltage_To_Pin(PAD_VDD1, High);
      //chip->Apply_Voltage_To_Pin(PAD_VDD2, High);
      //chip->Apply_Voltage_To_Pin(GND, Low);
      //chip->Apply_Voltage_To_Pin(PAD_GND1, Low);
      //chip->Apply_Voltage_To_Pin(PAD_GND2, Low);
      chip->Apply_Voltage_To_Pin(CLK_A, Low);
      chip->Apply_Voltage_To_Pin(CLK_B, Low);
      chip->Apply_Voltage_To_Pin(ENCONF_H, Low);
      chip->Apply_Voltage_To_Pin(CONFOPT_H, Low);
      chip->Update(delays);

      //A-latch
      chip->Apply_Voltage_To_Pin(CLK_A, High);
      if(bit == '1')
	chip->Apply_Voltage_To_Pin(CONFIN_H, High);
      else
	chip->Apply_Voltage_To_Pin(CONFIN_H, Low);
      chip->Update(delays);

      //Dead time
      chip->Apply_Voltage_To_Pin(CLK_A, Low);
      chip->Update(delays);

      //B-latch
      chip->Apply_Voltage_To_Pin(CLK_B, High);
      chip->Apply_Voltage_To_Pin(ENCONF_H, High);
      chip->Update(delays);

      //Finish
      chip->Apply_Voltage_To_Pin(CLK_B, Low);
      chip->Apply_Voltage_To_Pin(ENCONF_H, Low);
      chip->Update(delays);
    }
  else
    {
      //must be a binary digit or space
      if((bit != ' ')&&(bit != '\t')&&(bit != '\n'))
	{
	  error = NOT_A_BIT;
	  print_error(error, lines);
	} //if
    } //else
} //check_and_shift_config

///////////////////////////////////////////////////////////////////////////////
//End implementation file







