//interface.h: header file for sim-crowbar interface

#if !defined(_interface_h_)
#define _interface_h_

#include "block_array.h"
#include "signal.h"


///////////////////////////////////////////////////////////////////////////////
//scanner definitions

typedef int counter_type; //for counting lines of input

typedef enum token_type
{ 
  QUIT, ERROR, ASSIGN_PIN, ASSIGN_OPTICAL, SIMULATE, POS_INT,
  HELP, NAME, PRINT, COMMENT, END, YES, NO, CONFIG_STRING, ABOUT
}; //token_type

typedef union token_data_type {
  int ival;   //POS_INT
  char* name; //NAME
  char command; //ASSIGN_PIN
  char* config; //string of characters (hopefully 1's, 0's, and space)
} token_data; //union

//parser definitions
typedef enum state_type
{
  START_STATE, PROMPT_STATE, WAIT_FOR_COMMAND, WAIT_FOR_PIN,
  WAIT_FOR_ANOTHER_PIN, WAIT_FOR_OPTIC, WAIT_FOR_GATE_DELAYS,
  WAIT_FOR_SENSOR, WAIT_FOR_ANOTHER_SENSOR, WAIT_FOR_OPTION, 
  WAIT_FOR_ANOTHER_OPTION, WAIT_FOR_EOL, 
  DISCARD_LINE, ERROR_STATE, HELP_STATE, CONFIRM_QUIT, END_STATE, 
  WAIT_FOR_RESPONSE, CONFIGURE, ABOUT_STATE
}; //state_type

typedef enum parse_error_type
{
  UNDEFINED_COMMAND, INVALID_PIN, INVALID_OPTIC, INVALID_GATE_DELAYS,
  INVALID_NAME, INVALID_OPTION, EOL_EXPECTED, INVALID_RESPONSE, NONE
}; //parse_error_type

typedef enum semantic_error_type
{
  PIN_NUMBER_OUT_OF_RANGE, INTENSITY_OUT_OF_RANGE, UNKNOWN_OPTION,
  CSB_NUMBER_OUT_OF_RANGE, NOT_A_BIT
}; //semantic_error_type

///////////////////////////////////////////////////////////////////////////////
//parser prototypes

state_type next_state(token_type token, token_data data, state_type old_state);
//state machine (for simple parser)

void print_intro();

void print_error(parse_error_type code, counter_type line_number);

void prompt_for_input(counter_type line_number);

void prompt_for_confirmation();

void print_about();

void print_help();

///////////////////////////////////////////////////////////////////////////////
//semantic prototypes

void check_and_assign_pin(char* name, voltage_type voltage);


void check_and_assign_pin(int number, voltage_type voltage);


void check_and_print_data(char* option);


void check_and_simulate(unsigned int gate_delays);


void check_and_assign_optical(char* name, optical_type optic);


void check_and_assign_optical(int number, optical_type optic);


void print_error(semantic_error_type code, counter_type line_number);


void check_and_shift_config(char bit);


///////////////////////////////////////////////////////////////////////////////
//globals

static counter_type lines;
static CROWBAR_class *chip = new(CROWBAR_class); //actual chip model

///////////////////////////////////////////////////////////////////////////////
#endif







