//primitives.h: A header file defining useful primitives
//logic, latches

#if !defined(_Primitives_h_)
#define _Primitives_h_
#include "signal.h"

//Gates: no memory, simple functions of the inputs (AND, OR, NOT)
/////////////////////////////////////////////////////////////////
//1 input gates
signal_type INV(signal_type input);
signal_type BUF(signal_type input);

//2 input gates (fast implementation)
signal_type OR2(signal_type inp0, signal_type inp1);
signal_type NOR2(signal_type inp0, signal_type inp1);
signal_type AND2(signal_type inp0, signal_type inp1);
signal_type NAND2(signal_type inp0, signal_type inp1);

//multiple input gates
signal_type ORN(signal_type* inputs, unsigned int number_of_inputs);
signal_type NORN(signal_type* inputs, unsigned int number_of_inputs);
signal_type ANDN(signal_type* inputs, unsigned int number_of_inputs);
signal_type NANDN(signal_type* inputs, unsigned int number_of_inputs);

//Latches: simple memory elements
//Can be aggregated to form flops in derived classes
////////////////////////////////////////////////////////////////
class latch
{
	//defines a simple phase latch
	public:
		//public functions
		latch();
		~latch();
		void Update(signal_type clock, signal_type input);
		void Clear(); //initialize the latch
		signal_type Get_Output();
	protected:
		//protected contents
		signal_type latch_state;
	private:
		//nada
}; //latch

///////////////////////////////////////////////////////////////
//End of header file (create another file for derived cells)
#endif
